class InvalidApcResponse(Exception):
    def __init__(self, response):
        self.response = response

    def __str__(self):
        return "APC Response:\n status_code: {0} \n text: {1}".format(
            self.response.status_code, self.response.text
        )

    @classmethod
    def new(cls, response):
        """
        Create a new InvalidApcResponse exception based off the provided response.

        @param response: The HTTP response from requests.
        @return: A new InvalidApcResponse instance.
        """

        subclass_map = {
            ApcBadRequestError.status_code: ApcBadRequestError,
            ApcUnauthorizedError.status_code: ApcUnauthorizedError,
            ApcRequestTooLargeError.status_code: ApcRequestTooLargeError,
            ApcInternalServerError.status_code: ApcInternalServerError,
            ApcUnavailable.status_code: ApcUnavailable,
        }

        subclass = subclass_map.get(response.status_code)
        if subclass:
            return subclass(response)
        else:
            return cls(response)


class ApcBadRequestError(InvalidApcResponse):
    status_code = 400


class ApcUnauthorizedError(InvalidApcResponse):
    status_code = 401


class ApcRequestTooLargeError(InvalidApcResponse):
    status_code = 413


class ApcRateLimitingError(InvalidApcResponse):
    status_code = 429


class ApcInternalServerError(InvalidApcResponse):
    status_code = 500


class ApcUnavailable(InvalidApcResponse):
    status_code = 502

    # Do not use response.text because APC returns HTML on 502.
    def __str__(self):
        return "APC Response:\n502 Bad Gateway"
