from tic.shared.utils.util import message_chunk_batch


class FirehoseClient(object):
    def __init__(self, boto_session, stream_name, boto_config=None, endpoint_url=None):
        """

        @param boto_session: Pre-existing boto session
        @param stream_name: A name to identify the stream
        @param boto_config: Optional boto config for the firehose client
        """

        self.stream_name = stream_name
        self.firehose_client = boto_session.client(
            "firehose", config=boto_config, endpoint_url=endpoint_url
        )

    def put_record(self, record):
        """
        Writes a single data record into to the firehose delivery stream

        @param record: The record to be written
        """
        self.firehose_client.put_record(
            DeliveryStreamName=self.stream_name, Record=record
        )

    def put_record_batch(self, records, num_records_per_batch=500):
        """
        Writes multiple data records into a firehose delivery stream in a single call

        @param records: List of one or more records
        @param num_records_per_batch: The max number of records for one put_records call
        """

        # Batch based on kinesis client put_records limit of 500 records or 4 MB limit
        for batch in message_chunk_batch(
            records, num_records_per_batch, max_message_size=4191304, delimiter="\n"
        ):

            firehose_records = []
            for message in batch:
                firehose_records.append({"Data": message})

            self.firehose_client.put_record_batch(
                DeliveryStreamName=self.stream_name, Records=firehose_records
            )
