import json

from tic.shared.file_acquisition.dtos.base_job import BaseJob


class ExtractorJob(BaseJob):
    def __init__(
        self,
        sha256: str,
        source_name: str,
        source_id: int,
        package_name: str,
        package_s3_key: str,
    ):
        super().__init__(sha256, source_name, source_id, package_name)
        self.package_s3_key = package_s3_key

    @staticmethod
    def parse(message: str):
        y = json.loads(message)
        sha256 = y["sha256"]
        source_name = y["source_name"]
        source_id = y["source_id"]
        package_name = y["package_name"]
        package_s3_key = y["package_s3_key"]

        job = ExtractorJob(sha256, source_name, source_id, package_name, package_s3_key)
        return job

    @staticmethod
    def get_s3_key_for_data_file(sha256: str, file_name: str):
        """
        Generate s3 key based on first four letters of package sha256
        @param sha256: package sha256
        @param file_name: name of the data file
        @return: S3 key for the package
        """
        return f"metadata/{sha256[:2]}/{sha256[2:4]}/{file_name}"
