import pathlib

from magic import magic

from tic.shared.file_acquisition.os_util import CommonUtil


# noinspection SpellCheckingInspection
class FileTypes:
    def __init__(self):
        self.filemagic = magic.Magic(mime=True)

    archive_mime_types = [
        "application/vnd.ms-cab-compressed",
        "application/x-gzip",
        "application/x-rpm",
        "application/x-xar",
        "application/zip",
        "application/x-ms-installer",
        "application/x-msi",
        "application/x-windows-installer",
        "application/x-7z-compressed",
        "application/x-apple-diskimage",
        "application/x-bcpio",
        "application/x-bzip2",
        "application/x-bzip",
        "application/x-debian-package",
    ]

    pe_exe_mime_types = [
        "application/x-executable",
        "application/x-msdos-program",
        "application/x-dosexec",
        "application/x-dosdriver",
        "application/exe",
        "application/x-exe",
        "application/x-msdownload",
        "application/x-msi",
        "application/x-ms-installer",
        "application/x-sharedlib",
        "application/vnd.microsoft.portable-executable",
        "application/x-windows-installer",
    ]

    archive_file_extensions = [
        ".7z",
        ".bz",
        ".bz2",
        ".cab",
        ".cpio",
        ".deb",
        ".dmg",
        ".exe",
        ".iso",
        ".msi",
        ".msu",
        ".msp",
        ".pkg",
        ".rar",
        ".rpm",
        ".tar",
        ".wim",
        ".zip",
    ]

    def is_archive_file(self, package_path):
        ext = pathlib.Path(package_path).suffix

        if ext in FileTypes.archive_file_extensions:
            return True

        if ext in CommonUtil.file_extensions_to_ignore:
            return False

        file_type = self.get_file_type(package_path)
        if file_type in FileTypes.archive_mime_types:
            return True
        file_name = pathlib.Path(package_path).stem

        # special case of mac package files
        if file_name in ["Payload~", "Scripts~"]:
            return True

        return False

    def get_exe_or_archive_info(self, file_path):
        ext = FileTypes.get_file_extension(file_path)

        if ext in CommonUtil.file_extensions_to_ignore:
            return False, ext, None

        if file_path.endswith(".resources.dll") or file_path.endswith(".Resources.dll"):
            return False, ".resources.dll", None

        file_type = self.get_file_type(file_path)

        if file_type in CommonUtil.exe_mime_types:
            return True, ext, file_type

        if ext in CommonUtil.exe_file_extensions:
            return True, ext, file_type

        if ext in FileTypes.archive_file_extensions:
            return True, ext, file_type

        if file_type is None:
            return False, ext, file_type

        if file_type in FileTypes.archive_mime_types:
            return True, ext, file_type

        return False, ext, file_type

    def get_file_type(self, package_path):
        file_type = None
        try:
            file_type = self.filemagic.from_file(package_path)
        except FileNotFoundError:
            pass
        except Exception as ex:
            print(f"Not able to extrcat file type from {package_path} - {ex}")
        return file_type

    @staticmethod
    def get_file_extension(package_path):
        ext = pathlib.Path(package_path).suffix
        if ext or len(ext) > 0:
            ext = ext.lower()
        return ext
