from io import BytesIO
from google.cloud import storage


class StorageClient:
    def __init__(self, project_id=None):
        self.gcp_storage_client = storage.Client(project=project_id)

    def upload_blob_from_input_bytes(
        self,
        bucket_name,
        input_bytes: BytesIO,
        destination_blob_name,
        content_type="binary/octet-stream",
    ):
        bucket = self.gcp_storage_client.get_bucket(bucket_name)
        blob = bucket.blob(destination_blob_name)
        input_bytes.seek(0)
        blob.upload_from_string(input_bytes.getvalue(), content_type)

    def download_file(self, bucket_name, object_name, checksum=None):
        bucket = self.gcp_storage_client.get_bucket(bucket_name)
        blob = bucket.blob(object_name)
        return blob.download_as_bytes(checksum=checksum)
