import json
from tic.shared.scoring.score_calulator_base import ScoreCalculatorBase
from tic.shared.utils.utcdatetime import get_utc_now_string


class ReputationScore(object):
    """
    Code representation for generating/updating a reputation score
    """

    def __init__(self, reputation_score_calculator, reputation_score=None):
        if reputation_score is None:
            self.score = ReputationScore._generate_empty_score()
        else:
            if not all(
                key in reputation_score
                for key in (
                    "value",
                    "confidence",
                    "mean",
                    "median",
                    "mode",
                    "stddev",
                    "range",
                    "parts",
                    "last_update",
                    "version",
                )
            ):
                raise ValueError(
                    "reputation_score argument does not conform to ReputationScore schema"
                )
            else:
                self.score = reputation_score
        if reputation_score_calculator is None or not isinstance(
            reputation_score_calculator, ScoreCalculatorBase
        ):
            raise TypeError(
                "reputation_score_calculator must be an instance of ScoreCalculatorBase"
            )
        self.reputation_score_calculator = reputation_score_calculator

    def add_score(self, score_part):
        part_name = score_part["name"]
        existing = [
            index
            for index, part in enumerate(self.score["parts"])
            if part["name"] == part_name
        ]
        if len(existing) != 0:
            self.score["parts"][existing[0]] = score_part
        else:
            self.score["parts"].append(score_part)
        self.reputation_score_calculator.calculate(self.score)
        return self

    def to_internal_json(self):
        return json.dumps(self.to_internal_score())

    def to_public_json(self):
        return json.dumps(self.to_public_score())

    def to_public_score(self):
        temp_score = dict.copy(self.score)
        temp_score["parts"] = [
            part
            for part in self.score["parts"]
            if not ("public" in part and part["public"] is False)
        ]
        return temp_score

    def to_internal_score(self):
        return dict.copy(self.score)

    @staticmethod
    def _generate_empty_score():
        temp_score = {
            "value": None,
            "confidence": None,
            "mean": None,
            "median": None,
            "mode": None,
            "range": None,
            "stddev": None,
            "parts": [],
            "last_update": get_utc_now_string(),
            "version": None,
        }
        return temp_score

    @staticmethod
    def from_dict(dictionary, reputation_score_calculator):
        return ReputationScore(
            reputation_score=dictionary,
            reputation_score_calculator=reputation_score_calculator,
        )

    @staticmethod
    def from_json(json_string, reputation_score_calculator):
        return ReputationScore.from_dict(
            json.loads(json_string), reputation_score_calculator
        )
