from tic.shared.dto import Dto
from tic.shared.ubs.enums.analysis.pipeline import UbsAnalysisPipeline
from tic.shared.ubs.enums.analysis.status import UbsAnalysisStatus
from .stage_summary import StageSummary

from typing import List


class PipelineSummary(Dto):
    decode_names = {
        "pipeline_version": "version",
        "pipeline_status": "status",
        "pipeline_name": "name",
        "pipeline_stages": "stages",
    }
    encode_names = {
        "version": "pipeline_version",
        "status": "pipeline_status",
        "name": "pipeline_name",
        "stages": "pipeline_stages",
    }

    decode_settings = {
        "name": lambda x: UbsAnalysisPipeline[x],
        "status": lambda x: UbsAnalysisStatus[x],
        "version": lambda x: tuple(int(y) for y in x.split(".")),
        "stages": lambda x: [StageSummary.from_dict(y, True, True) for y in x],
    }
    encode_settings = {
        "name": lambda x: x.name,
        "status": lambda x: x.name,
        "version": lambda x: ".".join((str(y) for y in x)),
        "stages": lambda x: [y.to_dict(True, True, {None, ""}) for y in x],
    }

    def __init__(
        self,
        sha256: str = None,
        name: UbsAnalysisPipeline = None,
        status: UbsAnalysisStatus = UbsAnalysisStatus.NOT_STARTED,
        version: tuple = None,
        stages: List[StageSummary] = None,
    ):
        self.sha256 = sha256
        self.name = name
        self.status = status
        self.version = version
        self.stages = stages if stages is not None else list()
