from ...ubs.util import hash_to_binary
from ...ubs.util import signature_values


class IndexCustomerMetadata(object):
    def __init__(self, sha256, md5, size, org_key):
        self.sha256 = sha256
        self.md5 = md5
        self.size = size
        self.org_key = org_key

    def to_tuple(self):
        return (
            hash_to_binary(self.sha256),
            hash_to_binary(self.md5),
            self.size,
            self.org_key,
        )


class SignatureData(object):
    def __init__(
        self,
        sha256,
        org_key,
        publisher_name,
        issuer_name,
        first_seen,
        sign_timestamp,
        sign_state,
        num_occurrences,
    ):
        self.sha256 = sha256
        self.org_key = org_key
        self.publisher_name = publisher_name
        self.issuer_name = issuer_name
        self.first_seen = first_seen
        self.sign_timestamp = sign_timestamp
        self.sign_state = (
            signature_values[sign_state] if isinstance(sign_state, str) else sign_state
        )
        self.num_occurrences = num_occurrences

    def to_tuple(self):
        return (
            hash_to_binary(self.sha256),
            self.org_key,
            self.publisher_name,
            self.issuer_name,
            self.first_seen,
            self.sign_timestamp,
            self.sign_state,
            self.num_occurrences,
        )

    def __hash__(self):
        return hash(
            (
                self.sha256,
                self.org_key,
                self.publisher_name,
                self.issuer_name,
                self.sign_timestamp,
                self.sign_state,
            )
        )
