"""
This is taken directly from the cballiance repository - RJPECK
"""

import logging

_logger = logging.getLogger(__name__)


class DataSharing(object):
    who_mask = {"BIT9": 1, "VIRUSTOTAL": 2, "THIRDPARTY": 4}

    def __init__(self, hash=0, binary=0):
        self.hash = hash
        self.binary = binary

    @classmethod
    def from_api(cls, share_with_who):
        sharing = cls()

        for what, who_list in share_with_who.items():
            if what == "HASH":
                for who in who_list:
                    sharing.hash |= cls.get_mask(who)
            elif what == "BIN":
                for who in who_list:
                    sharing.binary |= cls.get_mask(who)
            else:
                _logger.debug("Unknown sharing scope: %s", what)

        return sharing

    @classmethod
    def from_model(cls, record):
        if record is None:
            return cls()
        return cls(record.hash_sharing, record.binary_sharing)

    def to_model(self, record):
        record.hash_sharing = self.hash
        record.binary_sharing = self.binary

    def to_db_update(self):
        return {"hash_sharing": self.hash, "binary_sharing": self.binary}

    def hash_enabled_for(self, who):
        return self._enabled_for(self.hash, who)

    def binary_enabled_for(self, who):
        return self._enabled_for(self.binary, who)

    @classmethod
    def get_mask(cls, *args):
        mask = 0
        for arg in args:
            bit = cls.who_mask.get(arg)
            if not bit:
                _logger.warn("Unknown sharing 'who': %s", arg)
            else:
                mask |= bit
        return mask

    def _enabled_for(self, what, who):
        mask = self.get_mask(who)

        return (what & mask) > 0

    def __eq__(self, other):
        return self.hash == other.hash and self.binary == other.binary

    def __ne__(self, other):
        return not self == other

    def __or__(self, other):
        return DataSharing(self.hash | other.hash, self.binary | other.binary)

    def __and__(self, other):
        return DataSharing(self.hash & other.hash, self.binary & other.binary)
