# Get an ELB for the service if it exists
from tic.shared.utils import kubernetes_util


def get_elb_service(service_name):
    service_name_lower = service_name.lower()
    if kubernetes_util.KUBERNETES_ENABLED:
        # If Running in kubernetes use kubernetes load balancer name
        if service_name_lower in kubernetes_util.SERVICE_TRANSLATION:
            return {
                "node": {
                    "Address": kubernetes_util.SERVICE_TRANSLATION[service_name_lower]
                },
                "service": {"Port": "10000"},
            }
        else:
            raise Exception("Service not found")


def get_healthy_service_uri(service_name, api_version, *args):
    service_instance = get_elb_service(service_name)

    if service_instance:
        service_uri = "http://{0}:{1}/{2}/".format(
            service_instance["node"]["Address"],
            service_instance["service"]["Port"],
            api_version,
        ) + "/".join(part for part in args)
        return service_uri
    return None
