import re
from enum import Enum


_md5_regex = re.compile(r"^[A-Fa-f\d]{32}$")
_sha1_regex = re.compile(r"^[A-Fa-f\d]{40}$")
_sha256_regex = re.compile(r"^[A-Fa-f\d]{64}$")
_sha512_regex = re.compile(r"^[A-Fa-f\d]{128}$")


class HashType(Enum):
    unknown = ""
    md5 = "md5"
    sha1 = "sha1"
    sha256 = "sha256"
    sha512 = "sha512"


def determine_hash_type(hash_value) -> (HashType, str):
    hash_type = HashType.unknown
    if hash_value is None:
        return hash_type, hash_value
    temp_hash = hash_value.strip()
    if _md5_regex.match(temp_hash):
        hash_type = HashType.md5
    elif _sha1_regex.match(temp_hash):
        hash_type = HashType.sha1
    elif _sha256_regex.match(temp_hash):
        hash_type = HashType.sha256
    elif _sha512_regex.match(temp_hash):
        hash_type = HashType.sha512

    return hash_type, temp_hash


def is_valid_md5(hash_value):
    """

    :param hash_value: Hash to check if it is a valid md5
    :return: True if valid - false if not.
    """
    return bool(_md5_regex.match(hash_value))


def is_valid_sha1(hash_value):
    """

    :param hash_value: Hash to check if it is a valid sha1
    :return: True if valid - False if not.
    """
    return bool(_sha1_regex.match(hash_value))


def is_valid_sha256(hash_value):
    """

    :param hash_value: Hash to check if it is a valid sha256
    :return: True if valid - false if not.
    """
    return bool(_sha256_regex.match(hash_value))


def is_valid_sha512(hash_value):
    """

    :param hash_value: Hash to check if it is a valid sha512
    :return: True if valid - false if not.
    """
    return bool(_sha512_regex.match(hash_value))


def is_not_valid_md5(hash_value_list):
    """
    Return invalid md5's from the specified hash_value_list
    :param hash_value_list:
    :return:
    """
    return [h for h in hash_value_list if not is_valid_md5(h)]
