# Program imports
from .enums.apc_verdict import ApcVerdict
from .enums.apc_category import ApcCategory
from .dtos.apc_category_description import ApcCategoryDescription


apc_category_map = {
    ApcCategory.UNKNOWN: ApcCategoryDescription(False, False, "No information"),
    ApcCategory.CLEAN: ApcCategoryDescription(False, False, "Clean"),
    ApcCategory.MALWARE: ApcCategoryDescription(True, True, "Malware"),
    ApcCategory.ADSPY: ApcCategoryDescription(
        True, True, "Adware with spying functions"
    ),
    ApcCategory.ADWARE: ApcCategoryDescription(True, True, "Adware"),
    ApcCategory.APPL: ApcCategoryDescription(
        False, False, "Application, not necessarily suspicious"
    ),
    ApcCategory.BAT: ApcCategoryDescription(True, True, "Batch Malware"),
    ApcCategory.BDC: ApcCategoryDescription(True, False, "Backdoor Client"),
    ApcCategory.BDS: ApcCategoryDescription(True, True, "Backdoor Server"),
    ApcCategory.BOO: ApcCategoryDescription(True, True, "Malicious Bootsector"),
    ApcCategory.DDOS: ApcCategoryDescription(
        True, True, "Distributed Denial-Of-Service malware"
    ),
    ApcCategory.DIAL: ApcCategoryDescription(False, False, "Dialer"),
    ApcCategory.DOS: ApcCategoryDescription(True, True, "DOS-based Malware"),
    ApcCategory.DR: ApcCategoryDescription(True, True, "Malware Dropper"),
    ApcCategory.EML: ApcCategoryDescription(
        True, True, "E-Mail which contains malicious content"
    ),
    ApcCategory.EXP: ApcCategoryDescription(True, True, "Exploit / Vulnerabilty"),
    ApcCategory.GAME: ApcCategoryDescription(False, False, "Game Program"),
    ApcCategory.HTML: ApcCategoryDescription(True, True, "HTML Script Malware"),
    ApcCategory.IRC: ApcCategoryDescription(True, True, "Internet Relay Chat Malware"),
    ApcCategory.JAVA: ApcCategoryDescription(True, True, "JAVA Malware"),
    ApcCategory.JOKE: ApcCategoryDescription(True, True, "Joke Application"),
    ApcCategory.JS: ApcCategoryDescription(True, True, "Javascript Malware"),
    ApcCategory.KIT: ApcCategoryDescription(True, False, "Malware Creation Kit"),
    ApcCategory.LINUX: ApcCategoryDescription(True, True, "Linux-based Malware"),
    ApcCategory.OSX: ApcCategoryDescription(True, True, "Apple OSX-based Malware"),
    ApcCategory.PERL: ApcCategoryDescription(True, True, "PERL Language Malware"),
    ApcCategory.PFS: ApcCategoryDescription(True, True, "Pontential Fake Software"),
    ApcCategory.PHISH: ApcCategoryDescription(
        True, False, "Phishing E-Mail or Webpage"
    ),
    ApcCategory.PHP: ApcCategoryDescription(True, True, "PHP Language Malware"),
    ApcCategory.RKIT: ApcCategoryDescription(True, True, "Rootkit"),
    ApcCategory.SPR: ApcCategoryDescription(True, False, "Security Privacy Risk Tool"),
    ApcCategory.SWF: ApcCategoryDescription(True, True, "Shockwave Flash Malware"),
    ApcCategory.SYMBOS: ApcCategoryDescription(True, True, "SymbianOS-based Malware"),
    ApcCategory.TR: ApcCategoryDescription(True, True, "Trojan Horse / Ransomware"),
    ApcCategory.UNIX: ApcCategoryDescription(True, True, "Linux-based Malware"),
    ApcCategory.VBS: ApcCategoryDescription(True, True, "Visual Basic Script Malware"),
    ApcCategory.W32: ApcCategoryDescription(
        True, True, "Windows 32bit Virus or File Infector"
    ),
    ApcCategory.W64: ApcCategoryDescription(
        True, True, "Windows 64bit Virus or File Infector"
    ),
    ApcCategory.WORM: ApcCategoryDescription(True, True, "Worm"),
    ApcCategory.W95: ApcCategoryDescription(True, True, "Windows95-based Malware"),
    ApcCategory.W2000: ApcCategoryDescription(True, True, "Windows2000-based Malware"),
    ApcCategory.HEUR: ApcCategoryDescription(
        True, True, "Suspicious content / Heuristic-based Malware"
    ),
    ApcCategory.PCK: ApcCategoryDescription(
        False, False, "Using unusual packing methods"
    ),
    ApcCategory.ANDROID: ApcCategoryDescription(True, True, "AndroidOS-based Malware"),
    ApcCategory.HIDDENEXT: ApcCategoryDescription(
        True, True, "Misleading file extension"
    ),
    ApcCategory.W2000M: ApcCategoryDescription(
        True, True, "Macrovirus for Office Word 2000"
    ),
    ApcCategory.W97M: ApcCategoryDescription(
        True, True, "Macrovirus for Office Word 97"
    ),
    ApcCategory.PUA: ApcCategoryDescription(
        True, False, "Potential Unwanted Application"
    ),
    ApcCategory.CLN: ApcCategoryDescription(
        False, False, "CLEAN (internal classification)"
    ),
    ApcCategory.VBA: ApcCategoryDescription(
        True, True, "Visual Basic Application Malware"
    ),
}


def get_apc_description(apc_category):
    """
    Given an ApcCategory enum, return the appropriate description DTO.

    @param apc_category: The ApcCategory to use.
    @return: An ApcCategoryDescription instance.
    """

    return apc_category_map[apc_category]


def get_apc_verdict(category_description):
    if category_description.malicious:
        return ApcVerdict.MALWARE
    elif category_description.blocked:
        return ApcVerdict.PUP
    else:
        return ApcVerdict.CLEAN
